@@ -66,14 +66,20 @@ test_that("megaEnv summary produces correct output", {
 test_that("varComp summary produces correct output", {
   geVCLm <- gxeVarComp(TD = BLUEs, trait = "t1", engine = "lme4")
   sumVCLm <- capture.output(summary(geVCLm))
-  expect_true("t1 ~ trial + (1 | genotype) " %in% sumVCLm)
-  expect_true("Sources of variation" %in% sumVCLm)
+  expect_true(" t1 ~ trial + (1 | genotype) " %in% sumVCLm)
+  expect_true("Sources of variation for fully random model:" %in% sumVCLm)
+  expect_true(" t1 ~ (1 | trial) + (1 | genotype) " %in% sumVCLm)
+  expect_true("Analysis of Variance Table for fully fixed model:" %in% sumVCLm)
+  expect_true(" t1 ~ trial + genotype " %in% sumVCLm)
 
   skip_on_cran()
   skip_on_ci()
   geVCAs <- gxeVarComp(TD = BLUEs, trait = "t1", engine = "asreml")
   sumVCAs <- capture.output(summary(geVCAs))
-  expect_true("t1 ~ trial + (1 | genotype) " %in% sumVCAs)
-  expect_true("Sources of variation" %in% sumVCAs)
+  expect_true(" t1 ~ trial + (1 | genotype) " %in% sumVCAs)
+  expect_true("Sources of variation for fully random model:" %in% sumVCAs)
+  expect_true(" t1 ~ (1 | trial) + (1 | genotype) " %in% sumVCAs)
+  expect_true("Analysis of Variance Table for fully fixed model:" %in% sumVCAs)
+  expect_true(" t1 ~ trial + genotype " %in% sumVCAs)
 })
 
